/*
 * Copyright 2006-2008 The FLWOR Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "api/collectionimpl.h"
#include <zorba/item.h>
#include <zorba/error_handler.h>
#include <istream>
#include "api/zorbaimpl.h"

#include <zorba/exception.h>

#include "zorbaerrors/errors.h"
#include "zorbaerrors/error_manager.h"
#include "zorbautils/latch.h"

#include "api/unmarshaller.h"

#include "system/globalenv.h"

#include "runtime/api/plan_wrapper.h"

#include "store/api/collection.h"
#include "store/api/item.h"
#include "store/api/store.h"


namespace zorba {

CollectionImpl::CollectionImpl(
    const store::Collection_t& aCollection,
    ErrorHandler* aErrorHandler)
  :
  theCollection(aCollection),
  theErrorHandler(aErrorHandler)
{
}


CollectionImpl::~CollectionImpl()
{
}

 
Item
CollectionImpl::getUri() 
{
  ZORBA_TRY
    return theCollection->getUri();
  ZORBA_CATCH
  return Item();
}

 
bool
CollectionImpl::addNode(const Item& aNode) 
{
  ZORBA_TRY

    store::Item* lItem = Unmarshaller::getInternalItem(aNode);

    SYNC_CODE(AutoLatch(GENV_STORE.getGlobalLock(), Latch::WRITE);)

    theCollection->addToCollection(lItem);
    return true;

  ZORBA_CATCH
  return false;
}

 
bool
CollectionImpl::addNodes(const ResultIterator* aResultIterator) 
{
  ZORBA_TRY

    PlanWrapper_t lPlan = Unmarshaller::getInternalPlan(aResultIterator);

    SYNC_CODE(AutoLatch(GENV_STORE.getGlobalLock(), Latch::WRITE);)

    theCollection->addToCollection(&*lPlan);
    return true;

  ZORBA_CATCH
  return false;
}


bool
CollectionImpl::addDocument(std::istream& lInStream)
{
  ZORBA_TRY
    theCollection->addToCollection(lInStream);
    return true;
  ZORBA_CATCH
  return false;
}


bool
CollectionImpl::deleteNode(const Item& aNode) 
{
  ZORBA_TRY
    store::Item* lItem = Unmarshaller::getInternalItem(aNode);

    SYNC_CODE(AutoLatch(GENV_STORE.getGlobalLock(), Latch::WRITE);)

    theCollection->removeFromCollection(lItem);
    return true;
  ZORBA_CATCH
  return false;
}
 

} /* namespace zorba */
